<?php
/**
 * Piwik - Open source web analytics
 *
 * @link http://piwik.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 *
 * @category Piwik_Plugins
 * @package Groups
 */
namespace Piwik\Plugins\Groups;

use Exception;
use Piwik\Access;
use Piwik\Common;
use Piwik\Db;
use Piwik\Piwik;
use Piwik\Site;
use Piwik\Tracker\Cache;
use Piwik\Plugins\UsersManager\API as UsersManagerAPI;

/**
 * API for plugin Groups
 *
 * @method static \Piwik\Plugins\Groups\API getInstance()
 */
class API extends \Piwik\Plugin\API
{
    /**
    * @var Model
    */
    private $model;
     
    protected function __construct()
    {
        $this->model = new Model();
    }
    
    
    /**
    * Check if the group exists
    *
    * @param $groupId the group ID to check for
    */
    public function groupIdExists($groupId)
    {
        return in_array($groupId, $this->model->getGroups(true));
    }
    
    /**
    * Returns each group that the given user has access to.
    * If $userlogin is not provided or null, the current user will be used.
    * If the user doesn't have any access, the returned array will be an empty array.
    *
    * @return array The returned array, in this format:
    *     array(
    *         array(
    *           'id'      => 0,
    *           'name'    => "Group A",
    *           'maxsites'=> 0,
    *           'access'  => "view"
    *         ),
    *         groupid2 => array(
    *           'id'      => 1,
    *           'name'    => "Group B",
    *           'maxsites'=> 50,
    *           'access'  => "admin"
    *         ),
    *         ...
    *     )
    */
    public function getGroupsFromUser($user = null)
    {
        Piwik::checkUserIsNotAnonymous();
        if ($user == null)
        {
            $userLogin = Piwik::getCurrentUserLogin();
        } else
        {
            $userLogin = $user;
        }
        return $this->model->getGroupsFromUser($userLogin, false);
    }
    
    /**
    * Returns the group if the user has access to it, and False if they do not.
    * If $userlogin is not provided or null, the current user will be used.
    * If the user doesn't have any access, the returned array will be an empty array.
    *
    * @return array The returned array, in this format:
    *     array(
    *         array(
    *           'id'      => 0,
    *           'name'    => "Group A",
    *           'maxsites'=> 0,
    *           'access'  => "view"
    *         )
    *     )
    */
    public function getGroupFromUser($groupId, $user = null)
    {
        Piwik::checkUserIsNotAnonymous();
        if ($user == null)
        {
            $userLogin = Piwik::getCurrentUserLogin();
        } else
        {
            $userLogin = $user;
        }
        return $this->model->getGroupFromUser($groupId, $userLogin);
    }
    
    /**
    * Returns the list of members in this group. Each array item is a string with the user's login.
    *
    * @return array The returned array, in this format:
    *     array(
    *         "login1",
    *         "login2",
    *         ...
    *     )
    */
    public function getGroupMembers($groupId, $user = null)
    {
        if (!$this->isUserAllowedToAccessGroup($groupId))
        {
            throw new Exception(Piwik::translate('Groups_NoAccessGroup'));
        }
        return $this->model->getGroupMembers($groupId);
    }
    
    /**
    * Returns the list of sites in this group.
    *
    * @return array The returned array, in this format:
    *     array(
    *         siteId1 => "website1.example.com",
    *         siteId2 => "website2.example.com",
    *         ...
    *     )
    */
    public function getGroupSites($groupId, $user = null)
    {
        if (!$this->isUserAllowedToAccessGroup($groupId))
        {
            throw new Exception(Piwik::translate('Groups_NoAccessGroup'));
        }
        return $this->model->getGroupSites($groupId);
    }
    
    
    /**
    * Returns a flat array of group IDs that the user has access to
    * If $userlogin is not provided or null, the current user will be used.
    * If the user is a Super User, all groups will be returned.
    * If the user doesn't have any access, the returned array will be an empty array.
    *
    * @return array The returned array, in this format:
    *     array(
    *         groupid1,
    *         groupid2,
    *         ...
    *     )
    */
    public function getGroupIdsFromUser($user = null)
    {
        Piwik::checkUserIsNotAnonymous();
        if ($user == null)
        {
            $userLogin = Piwik::getCurrentUserLogin();
        } else
        {
            $userLogin = $user;
        }
        return $this->model->getGroupsFromUser($userLogin, true);
    }
    
    /**
    * Returns each group that the user is a member of
    * If $userlogin is not provided or null, the current user will be used.
    * Being a Super User has no effect here.
    * If the user doesn't have any access, the returned array will be an empty array.
    *
    * @return array The returned array, in this format:
    *     array(
    *         array(
    *           'id'      => 0,
    *           'name'    => "Group A",
    *           'maxsites'=> 0,
    *           'access'  => "view"
    *         ),
    *         groupid2 => array(
    *           'id'      => 1,
    *           'name'    => "Group B",
    *           'maxsites'=> 50,
    *           'access'  => "admin"
    *         ),
    *         ...
    *     )
    */
    public function getGroupsUserIsIn($user = null)
    {
        Piwik::checkUserIsNotAnonymous();
        if ($user == null)
        {
            $userLogin = Piwik::getCurrentUserLogin();
        } else
        {
            $userLogin = $user;
        }
        return $this->model->getGroupsFromUser($userLogin, false, true);
    }
    
    /**
    * Returns a flat array of group IDs that the user is a member of
    * If $userlogin is not provided or null, the current user will be used.
    * Being a Super User has no effect here.
    * If the user doesn't have any access, the returned array will be an empty array.
    *
    * @return array The returned array, in this format:
    *     array(
    *         groupid1,
    *         groupid2,
    *         ...
    *     )
    */
    public function getGroupIdsUserIsIn($user = null)
    {
        Piwik::checkUserIsNotAnonymous();
        if ($user == null)
        {
            $userLogin = Piwik::getCurrentUserLogin();
        } else
        {
            $userLogin = $user;
        }
        return $this->model->getGroupsFromUser($userLogin, true, true);
    }
    
    /**
    * Check whether the user is allowed to access the Groups plugin
    *
    * @return boolean The returned boolean represents whether the user is allowed to access the Groups plugin
    */
    public function isUserAllowedToAccessGroups()
    {
        return Piwik::hasUserSuperUserAccess() || ( count( $this->getGroupsFromUser() ) > 0 );
    }
    
    /**
    * Check whether the user is allowed to access a given group
    *
    * @return boolean The returned boolean represents whether the user is allowed to access the given group
    */
    public function isUserAllowedToAccessGroup($groupId)
    {
        return Piwik::hasUserSuperUserAccess() || ( in_array( $groupId, $this->getGroupIdsFromUser() ) );
    }
    
    /**
    * Check whether the user is allowed to modify a given group
    *
    * @return boolean The returned boolean represents whether the user is allowed to modify the given group
    */
    public function isUserAllowedToModifyGroup($groupId)
    {
        $group = $this->getGroupFromUser($groupId);
        return Piwik::hasUserSuperUserAccess() || $group['access'] == "admin";
    }
    
    /**
     * Check that the group name has a correct format.
     *
     * @param $groupName
     * @throws Exception
     */
    private function checkName($groupName)
    {
        if (empty($groupName))
        {
            throw new Exception(Piwik::translate("Groups_ExceptionEmptyName"));
        }
        if (strlen($groupName) > 100)
        {
            throw new Exception(Piwik::translate("Groups_ExceptionNameTooLong"));
        }
    }
    
    /**
    * Enforce that the user is allowed to access the Groups plugin
    *
    * @return boolean The returned boolean represents whether the user is allowed to access the Groups plugin
    */
    public function checkUserIsAllowedToAccessGroups()
    {
        if (!$this->isUserAllowedToAccessGroups())
        {
            throw new Exception(Piwik::translate('Groups_NoAccess'));
        }
    }
    
    /**
    * Enforce that the user is allowed to access a given group
    *
    * @param $groupId the ID of the group you want to enfore access for
    *
    * @throws Exception Groups_NoAccessGroup (You do not have access to this group) if user does not have access.
    */
    public function checkUserIsAllowedToAccessGroup($groupId)
    {
        if (!$this->isUserAllowedToAccessGroup($groupId))
        {
            throw new Exception(Piwik::translate('Groups_NoAccessGroup'));
        }
    }
    
    /**
    * Enforce that the user is allowed to modify a given group
    *
    * @param $groupId the ID of the group you want to enfore access for
    *
    * @throws Exception Groups_NoAccessGroup (You do not have access to this group) if user does not have access.
    */
    public function checkUserIsAllowedToModifyGroup($groupId)
    {
        if (!$this->isUserAllowedToModifyGroup($groupId))
        {
            throw new Exception(Piwik::translate('Groups_ExceptionYouAreNotAllowedModifyGroup'));
        }
    }
    
    /**
    * Enforce that the user exists
    *
    * @param $login the user to check for
    *
    * @throws Exception Groups_ExceptionThatUserDoesNotExist (That user does not exist) if they do not.
    */
    public function checkUserExists($login)
    {
        if (!UsersManagerAPI::getInstance()->userExists($login)) {
            throw new Exception(Piwik::translate('Groups_ExceptionThatUserDoesNotExist'));
        }
    }
    
    /**
    * Enforce that the user is in a group
    *
    * @param $login the user to check for
    * @param $groupId the group ID to check for
    *
    * @throws Exception Groups_ExceptionUserAlreadyInGroup (user already in group) if they are.
    */
    public function checkUserIsInGroup($login, $groupId)
    {
        $groupIds = $this->getGroupIdsUserIsIn($login);
        if (! in_array($groupId, $groupIds))
        {
            throw new Exception(Piwik::translate('Groups_ExceptionUserNotInGroup'));
        }
    }
    
    /**
    * Enforce that the user is not in a group
    *
    * @param $login the user to check for
    * @param $groupId the group ID to check for
    *
    * @throws Exception Groups_ExceptionUserAlreadyInGroup (user already in group) if they are.
    */
    public function checkUserIsNotInGroup($login, $groupId)
    {
        $groupIds = $this->getGroupIdsUserIsIn($login);
        if (in_array($groupId, $groupIds))
        {
            throw new Exception(Piwik::translate('Groups_ExceptionUserAlreadyInGroup'));
        }
    }
    
    
    /**
    * Enforce that the group exists
    *
    * @param $groupId the group ID to check for
    *
    * @throws Exception Groups_ExceptionThatGroupDoesNotExist (That group does not exist) if it does not.
    */
    public function checkGroupIdExists($groupId)
    {
        if (!$this->groupIdExists($groupId)) {
            throw new Exception(Piwik::translate('Groups_ExceptionThatGroupDoesNotExist'));
        }
    }
    
    
    /**
    * GROUP CRUD Functions
    */
    
    /**
    * Add a group.
    * Requires Super User access.
    *
    * @param string $groupName name of the group
    * @param int $maxSites Maximum number of sites allowed for this group
    */
    public function addGroup($groupName, $maxSites)
    {
        Piwik::checkUserHasSuperUserAccess();

        $this->checkName($groupName);
        
        $maxSites = (int)$maxSites;
        
        $groupId = $this->model->addGroup($groupName, $maxSites);

        Piwik::postEvent('Groups.addGroup.end', array($groupId));
        
        return $groupId;
    }
    
    /**
    * Update an existing group.
    * Requires group admin or Super User access.
    *
    * @param int $groupId ID of the group to edit
    * @param string $groupName name of the group
    * @param int $maxSites Maximum number of sites allowed for this group
    */
    public function updateGroup($groupId, $groupName = null, $maxSites = null)
    {
        $this->checkUserIsAllowedToModifyGroup($groupId);
        $this->checkGroupIdExists($groupId);

        $this->model->updateGroup($groupId, $groupName, $maxSites);
        
        Piwik::postEvent('Groups.updateGroup.end', array($groupId));
    }
    
    /**
    * Delete a group from the database, given its Id.
    * Requires group admin or Super User access.
    *
    * @param int $idSite
    */
    public function deleteGroup($groupId)
    {
        $this->checkUserIsAllowedToModifyGroup($groupId);
        $this->checkGroupIdExists($groupId);
        
        $this->model->deleteGroup($groupId);

        Piwik::postEvent('Groups.deleteGroup.end', array($groupId));
    }
    
    
    
    /**
    * GROUP MEMBER CRUD Functions
    */
    
    /**
    * Add a user to the group.
    * Requires group admin or Super User access.
    *
    * @param int $groupId the ID of the group
    * @param string $login The username to add
    */
    public function addMember($groupId, $login, $access)
    {
        $this->checkUserIsAllowedToModifyGroup($groupId);
        $this->checkGroupIdExists($groupId);
        $this->checkUserExists($login);
        $this->checkUserIsNotInGroup($login, $groupId);
        $this->model->addMember($groupId, $login, $access);

        Piwik::postEvent('Groups.addMember.end', array($groupId, $login));
        
    }
    
    /**
    * Update a member's access level (admin or view).
    * Requires group admin or Super User access.
    *
    * @param int $groupId ID of the group to edit
    * @param string $groupName name of the group
    * @param int $maxSites Maximum number of sites allowed for this group
    */
    public function updateMember($groupId, $login, $access = null)
    {
        $this->checkUserIsAllowedToModifyGroup($groupId);
        $this->checkGroupIdExists($groupId);
        $this->checkUserExists($login);
        $this->checkUserIsInGroup($login, $groupId);
        
        if (!is_null($access))
        {
            if ($access != "admin")
            {
                $access = "view";
            }
        }
        
        $this->model->updateMember($groupId, $login, $access);
        
        Piwik::postEvent('Groups.updateMember.end', array($groupId, $login));
        
    }
    
    /**
    * Remove a user from a group.
    *
    * Requires Super User access.
    *
    * @param int $idSite
    */
    public function deleteMember($groupId, $login)
    {
        $this->checkUserIsAllowedToModifyGroup($groupId);
        $this->checkUserExists($login);
        $this->checkUserIsInGroup($login, $groupId);
        
        $this->model->deleteMember($groupId, $login);
       
        Piwik::postEvent('Groups.deleteMember.end', array($groupId, $login));
    }

}
