<?php
/**
<?php
/**
 * Piwik - Open source web analytics
 *
 * @link http://piwik.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 *
 * @category Piwik_Plugins
 * @package Groups
 */
namespace Piwik\Plugins\Groups;

use Piwik\Common;
use Piwik\Db;
use Piwik\Piwik;

class Model
{
    
    /**
    * @var \Piwik\Db
    */
    private $db;

    public function __construct()
    {
        $this->db = Db::get();
    }
    
    /**
    * Returns all groups.
    *
    * @return array The returned array has this format if $ids is false:
    *     array(
    *         array(
    *           'id'      => 0,
    *           'name'    => "Group A",
    *           'maxsites'=> 0,
    *           'access'  => "view"
    *         ),
    *         groupid2 => array(
    *           'id'      => 1,
    *           'name'    => "Group B",
    *           'maxsites'=> 50,
    *           'access'  => "admin"
    *         ),
    *         ...
    *     )
    * and this format if $ids is true:
    *     array(
    *         groupid1,
    *         groupid2,
    *         ...
    *     )
    */
    public function getGroups($ids = false)
    {
        $sql = "SELECT id, name, maxsites, access 
                                      FROM      " . Common::prefixTable("group")
                                  . " LEFT JOIN " . Common::prefixTable("group_access")
                                  . " ON group_id = id";
        $groups = $this->db->fetchAll($sql);
        $groups_2 = array();
        foreach ($groups as $key => $group)
        {
            $groups_2[$group['id']] = $group;
        }
        $groups = $groups_2;
        if ($ids) {
            foreach ($groups as $key => $g) {
                $groups[$key] = $g['id'];
            }
        }
        return $groups;
    }
    
    /**
    * Returns each group that the given user has access to.
    * If the user doesn't have any access, the returned array will be an empty array.
    *
    * @param string $userLogin User that has to be valid, or null
    *
    * @return array The returned array has this format if $ids is false:
    *     array(
    *         array(
    *           'id'      => 0,
    *           'name'    => "Group A",
    *           'maxsites'=> 0,
    *           'access'  => "view"
    *         ),
    *         groupid2 => array(
    *           'id'      => 1,
    *           'name'    => "Group B",
    *           'maxsites'=> 50,
    *           'access'  => "admin"
    *         ),
    *         ...
    *     )
    * and this format if $ids is true:
    *     array(
    *         groupid1,
    *         groupid2,
    *         ...
    *     )
    */
    public function getGroupsFromUser($userLogin, $ids, $ignoreSuperUser = false)
    {
        $sql = "SELECT id, name, maxsites, access 
                                      FROM      " . Common::prefixTable("group")
                                  . " LEFT JOIN " . Common::prefixTable("group_access")
                                  . " ON group_id = id";
        if ( $ignoreSuperUser || ! Piwik::hasTheUserSuperUserAccess($userLogin) )
        {
            $sql .= " WHERE login = ?";
        }
        $groups = $this->db->fetchAll($sql, $userLogin);
        $groups_2 = array();
        foreach ($groups as $key => $group)
        {
            $groups_2[$group['id']] = $group;
        }
        $groups = $groups_2;
        if ($ids)
        {
            foreach ($groups as $key => $g)
            {
                $groups[$key] = $g['id'];
            }
        }
        return $groups;
    }
    /**
    * Returns each group that the given user has access to.
    * If the user doesn't have any access, the returned array will be an empty array.
    *
    * @param string $userLogin User that has to be valid, or null
    *
    * @return array The returned array has this format if $ids is false:
    *     array(
    *         array(
    *           'id'      => 0,
    *           'name'    => "Group A",
    *           'maxsites'=> 0,
    *           'access'  => "view"
    *         ),
    *         groupid2 => array(
    *           'id'      => 1,
    *           'name'    => "Group B",
    *           'maxsites'=> 50,
    *           'access'  => "admin"
    *         ),
    *         ...
    *     )
    * and this format if $ids is true:
    *     array(
    *         groupid1,
    *         groupid2,
    *         ...
    *     )
    */
    public function getGroupFromUser($groupId, $userLogin)
    {
        $sql = "SELECT id, name, maxsites, access 
                                      FROM      " . Common::prefixTable("group")
                                  . " LEFT JOIN " . Common::prefixTable("group_access")
                                  . " ON group_id = id"
                                  . " WHERE id = ?";
        $bind = array(
            (int)$groupId
        );
        if ( ! Piwik::hasTheUserSuperUserAccess($userLogin) )
        {
            $sql .= " AND login = ?";
            $bind[] = $userLogin;
        }
        $sql .= " LIMIT 1";
        
        $groups = $this->db->fetchAll($sql, $bind);
        return $groups[0];
    }
    
    
    /**
    * Returns the list of members in this group. Each array item is a string with the user's login.
    *
    * @return array The returned array, in this format:
    *     array(
    *         "login1",
    *         "login2",
    *         ...
    *     )
    */
    public function getGroupMembers($groupId)
    {
        $sql = "SELECT login, access FROM " 
              . Common::prefixTable("group_access")
              . " WHERE group_id = ?";
        $members = $this->db->fetchAll($sql, (int)$groupId);
        return $members;
    }
    
    /**
    * Returns the list of sites in this group.
    *
    * @return array The returned array, in this format:
    *     array(
    *         siteId1 => "website1.example.com",
    *         siteId2 => "website2.example.com",
    *         ...
    *     )
    */
    public function getGroupSites($groupId)
    {
        return array(
            // TO DO
        );
    }
    
    
    /**
    * Groups
    */
    public function addGroup($groupName, $maxSites)
    {
        $bind = array('name' => $groupName,
                      'maxsites' => $maxSites);
        $this->db->insert(Common::prefixTable("group"), $bind);
        return $this->db->lastInsertId();
        
    }
    public function updateGroup($groupId, $groupName = null, $maxSites = null)
    {
        $bind = array();

        if (!is_null($groupName))
        {
            $this->checkName($groupName);
            $bind['name'] = $groupName;
        }
        if (!is_null($maxSites))
        {
            $maxSites = (int)$maxSites;
            $bind['maxsites'] = $maxSites;
        }
        
        $this->db->update(Common::prefixTable("group"),
            $bind,
            "id = $groupId"
        );
        
    }
    public function deleteGroup($groupId)
    {
        $this->db->query("DELETE FROM " . Common::prefixTable("group") . "
					WHERE id = ?", $groupId);

        $this->db->query("DELETE FROM " . Common::prefixTable("group_access") . "
					WHERE group_id = ?", $groupId);
    }
    
    
    /**
    * Group Members
    */
    public function addMember($groupId, $login, $access)
    {
        
        $bind = array('group_id' => $groupId,
                      'login' => $login,
                      'access' => $access);
        $this->db->insert(Common::prefixTable("group_access"), $bind);
    }
    public function updateMember($groupId, $login, $access = "view")
    {
        $bind = array("access"=>$access);
        $this->db->update(Common::prefixTable("group_access"),
            $bind,
            "group_id = $groupId AND login = '$login'"
        );
        
    }
    public function deleteMember($groupId, $login)
    {
        $this->db->query("DELETE FROM " . Common::prefixTable("group_access")
					. " WHERE group_id = ?"
					. " AND login = ?"
                    , array($groupId, $login));
    }
}
